package cn.ninegame.genericframework.basic;

import android.os.Message;
import android.util.Log;

/**
 * Created by zhonghui on 2017-04-17.
 */

final class BackgroundPoster implements Runnable {

    private final PendingPostQueue queue;
    private final MsgBroker msgBroker;

    private volatile boolean executorRunning;

    BackgroundPoster(MsgBroker msgBroker) {
        this.msgBroker = msgBroker;
        queue = new PendingPostQueue();
    }

    public void enqueue(Message subscription, GundamMessage event) {
        PendingPost pendingPost = PendingPost.obtainPendingPost(subscription, event);
        synchronized (this) {
            queue.enqueue(pendingPost);
            if (!executorRunning) {
                executorRunning = true;
                msgBroker.getExecutorService().execute(this);
            }
        }
    }

    @Override
    public void run() {
        try {
            try {
                while (true) {
                    PendingPost pendingPost = queue.poll(1000);
                    if (pendingPost == null) {
                        synchronized (this) {
                            // Check again, this time in synchronized
                            pendingPost = queue.poll();
                            if (pendingPost == null) {
                                executorRunning = false;
                                return;
                            }
                        }
                    }
                    msgBroker.invokeSubscriber(pendingPost);
                }
            } catch (InterruptedException e) {
                Log.w("Event", Thread.currentThread().getName() + " was interruppted", e);
            }
        } finally {
            executorRunning = false;
        }
    }

}
