package cn.ninegame.genericframework.basic;

import java.util.ArrayList;

import cn.ninegame.genericframework.module.ModuleInfo;
import cn.ninegame.genericframework.tools.StorageUtil;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * 框架处理类， 负责把框架的异常传递到应用层
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/2/17
 */
public class ExceptionManager {
    // 判断磁盘空间是否已满的临界值(单位: KB)
    private static final long THRESHOLD_MEMORY = 1024 * 200;

    private ArrayList<IExceptionHandler> mExceptionHandlerList;

    public ExceptionManager(){
        mExceptionHandlerList = new ArrayList<IExceptionHandler>();
    }

    /**
     * 添加异常处理类，可添加多个
     * @param exceptionHandler
     */
    public void addExceptionHandler(IExceptionHandler exceptionHandler){
        mExceptionHandlerList.add(exceptionHandler);
    }

    /**
     * 移除异常处理类
     * @param exceptionHandler
     */
    public void removeExceptionHandler(IExceptionHandler exceptionHandler){
        for (IExceptionHandler e : mExceptionHandlerList){
            if (e == exceptionHandler){
                mExceptionHandlerList.remove(e);
                return;
            }
        }
    }

    /**
     * 处理异常
     * @param e
     */
    public void handleException(Exception e){
        if (!mExceptionHandlerList.isEmpty()) {
            for (IExceptionHandler eh : mExceptionHandlerList){
                if (eh != null) {
                    eh.handleException(e);
                }
            }
        }
    }

    /**
     * 是否是磁盘已满异常
     * @return
     */
    public static boolean isDiskFullException(){
        long remainMemory = StorageUtil.getAvailInternalStorageSizeInKiloBytes();
        return remainMemory < THRESHOLD_MEMORY;
    }
}
