package cn.ninegame.genericframework.basic;

import cn.ninegame.genericframework.module.ILoadModuleListener;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: 框架配置</p>
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/3/23
 */
public class FrameworkConfig {
    //是否为debug模式
    public static boolean isDebug;
    //加解密
    private ISecurityDelegate securityDelegate;
    //缓存管理
    private ICacheDelegate cacheDelegate;
    //异常处理
    private IExceptionHandler exceptionHandler;

    private ILoadModuleListener loadModuleListener;

    private ILogHandler logHandler;

    //Environment api 拦截
    private EnvironmentInterceptor environmentInterceptor;

    /**
     * 是否打开栈调试工具
     */
    private boolean openStackDebugTool;

    /**
     * 打开全局动画开关， true则默认开启过场动画
     */
    private boolean openGlobalAnim;

    /**
     * 多层Fragment堆叠的时候， 只展示顶部两个Fragment， 其余的直接Hide
     */
    private boolean hideDeepFragment;

    public FrameworkConfig(){}

    private FrameworkConfig(Builder builder) {
        securityDelegate = builder.securityDelegate;
        cacheDelegate = builder.cacheDelegate;
        exceptionHandler = builder.exceptionHandler;
        loadModuleListener = builder.loadModuleListener;
        logHandler = builder.logHandler;
        environmentInterceptor = builder.environmentInterceptor;
        openStackDebugTool = builder.openStackDebugTool;
        openGlobalAnim = builder.openGlobalAnim;
        hideDeepFragment = builder.hideDeepFragment;
    }

    public boolean isDebug() {
        return isDebug;
    }

    public ISecurityDelegate getSecurityDelegate() {
        return securityDelegate;
    }

    public ICacheDelegate getCacheDelegate() {
        return cacheDelegate;
    }

    public IExceptionHandler getExceptionHandler() {
        return exceptionHandler;
    }

    public ILoadModuleListener getLoadModuleListener(){
        return loadModuleListener;
    }

    public ILogHandler getLogHandler(){ return logHandler; }

    public EnvironmentInterceptor getEnvironmentInterceptor() {
        return environmentInterceptor;
    }

    public boolean isOpenStackDebugTool() {
        return openStackDebugTool;
    }

    public boolean isOpenGlobalAnim() {
        return openGlobalAnim;
    }

    public boolean isHideDeepFragment() {
        return hideDeepFragment;
    }

    public static class Builder {
        private boolean isDebug;
        private ISecurityDelegate securityDelegate;
        private ICacheDelegate cacheDelegate;
        private IExceptionHandler exceptionHandler;
        private ILoadModuleListener loadModuleListener;
        private ILogHandler logHandler;
        private EnvironmentInterceptor environmentInterceptor;
        private boolean openStackDebugTool;
        private boolean openGlobalAnim;
        private boolean hideDeepFragment;
        private EnvironmentInterceptor environmentIntercepter;

        public Builder() {
        }

        public Builder setDebug(boolean debug) {
            isDebug = debug;
            return this;
        }

        public Builder setSecurityDelegate(ISecurityDelegate securityDelegate) {
            this.securityDelegate = securityDelegate;
            return this;
        }

        public Builder setCacheDelegate(ICacheDelegate cacheDelegate) {
            this.cacheDelegate = cacheDelegate;
            return this;
        }

        public Builder setExceptionHandler(IExceptionHandler exceptionHandler) {
            this.exceptionHandler = exceptionHandler;
            return this;
        }

        public Builder setLoadModuleListener(ILoadModuleListener loadModuleListener) {
            this.loadModuleListener = loadModuleListener;
            return this;
        }

        public Builder setLogHandler(ILogHandler val) {
            logHandler = val;
            return this;
        }

        public Builder setEnvironmentInterceptor(EnvironmentInterceptor val) {
            environmentInterceptor = val;
            return this;
        }

        public Builder setOpenStackDebugTool(boolean val) {
            openStackDebugTool = val;
            return this;
        }

        public Builder setOpenGlobalAnim(boolean val) {
            openGlobalAnim = val;
            return this;
        }

        public Builder setHideDeepFragment(boolean val) {
            hideDeepFragment = val;
            return this;
        }

        public FrameworkConfig build() {
            return new FrameworkConfig(this);
        }

        public Builder setLogHandlerListener(ILogHandler logHandler) {
            this.logHandler = logHandler;
            return this;
        }

        public Builder setEnvironmentIntercepter(EnvironmentInterceptor envIntercepter) {
            this.environmentIntercepter = envIntercepter;
            return this;
        }

        public FrameworkConfig create() {
            return new FrameworkConfig(this);
        }
    }
}
