package cn.ninegame.genericframework.basic;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * 框架安全相关的加解密、文件校验
 *
 * 注：下面方法名中的 static 表示数据是打包时候就已经存在的数据； dynamic 表示 打包以后动态生成的数据
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/3/1
 */
public interface ISecurityDelegate {

    /**
     * 静态字符串数据加密
     * @param data
     * @return
     */
    public String staticEncryptString(String data);

    /**
     * 静态字符串数据解密
     * @param data
     * @return
     */
    public String staticDecryptString(String data);

    /**
     * 动态字符串数据加密
     * @param data
     * @return
     */
    public String dynamicEncryptString(String data);

    /**
     * 动态字符串数据解密
     * @param data
     * @return
     */
    public String dynamicDecryptString(String data);

    /**
     * 动态字节数据加密
     *
     * 注意： 数据有可能会稍，需要使用效率较高的加解密方式
     * @param data
     * @return
     */
    public byte[] dynamicEncryptByte(byte[] data);

    /**
     * 动态字节解密
     * @param data
     * @return
     */
    public byte[] dynamicDecryptByte(byte[] data);

    /**
     * 检查apk的特定文件是否被篡改
     * @param path 待校验的文件路径
     * @return true为成功，false为失败
     */
    public boolean checkStaticFileValid(String path);

    /**
     * 生成并保存文件的签名
     * 文件修改以后需要重新初始化
     * 使用 {@link #isDynamicFileValid(String)} 检查文件是否被篡改
     * @param path 待初始化状态的文件路径
     * @return true为成功，false为失败
     */
    public boolean initDynamicFileSignature(String path);

    /**
     * 检查文件是否被篡改
     *
     * 注意：使用前必须先初始化文件状态 {@link #initDynamicFileSignature(String)}
     * @param path 待校验的文件路径
     *
     * @return true为成功，false为失败
     */
    public boolean isDynamicFileValid(String path);

    /**
     * 动态数据存储
     *
     * 注意：需要加密的敏感数据才使用该方法
     *
     * @param key
     * @param value
     * @return true为成功，false为失败
     */
    public boolean putDynamicString(String key, String value);

    /**
     * 获取动态数据
     * @param key
     * @return 长度不为0的字符串，失败时返回空。
     */
    public String getDynamicString(String key);

    /**
     * 该设备是否已经获得root权限
     * @return
     */
    public boolean isRoot();
}
