package cn.ninegame.genericframework.ui;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.annotation.Nullable;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;

import android.support.v4.app.FragmentHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import cn.ninegame.genericframework.basic.Environment;
import cn.ninegame.genericframework.basic.FrameworkFacade;
import cn.ninegame.genericframework.basic.FrameworkService;
import cn.ninegame.genericframework.basic.StatManager;
import cn.ninegame.genericframework.tools.LOG;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2015-03-19
 * Author      : lihq@ucweb.com
 * Description : Tell me what does this class do
 * ****************************************************************************
 */
public class BaseActivity extends AppCompatActivity implements FragmentManager.OnBackStackChangedListener {

    public final static String INTENT_EXTRA_FRAGMENT_TAG = "ftag";
    public final static String INTENT_EXTRA_LAUNCHER_MODE = "launcherMode";

    protected final static String SAVE_INSTANCE_BUNDLE_KEY_FRAGMENTS = "save_instance_fragment_ids";
    private static final String TAG = "BaseActivity";

    private Environment mEnv = null;
    private boolean mIsForeground = false;
    private boolean mIsResumed = false;

    private Handler mHandler;

    public final static String DIALOG_FRAGMENT_TAG = "dialog";

    public void setEnvironment(Environment env) {
        mEnv = env;
    }

    public Environment getEnvironment() {
        return  mEnv;
    }

    public boolean isForeground(){
        return mIsForeground;
    }

    protected void verifyModuleState(Bundle savedInstanceState) {
        return;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        if(mEnv == null){
            mEnv = FrameworkFacade.getInstance().getEnvironment();
        }
        if (mEnv != null) {
            mEnv.pushToActivityStack(this);
        }
        if (savedInstanceState != null) {
            restoreModulesState(savedInstanceState);
            verifyModuleState(savedInstanceState);
        }
        super.onCreate(savedInstanceState);
        handleIntent(getIntent());
        //注册监听BackStack变化
        getSupportFragmentManager().addOnBackStackChangedListener(this);
    }

    @Override
    protected void onResume() {
        super.onResume();

        mIsResumed = true;

        if(mEnv == null){
            mEnv = FrameworkFacade.getInstance().getEnvironment();
        }
        if (mEnv != null) {
            mEnv.pushToActivityStackIfNeed(this);
        }
    }

    @Override
    protected void onPause() {
        super.onPause();

        mIsResumed = false;
    }

    public boolean isPerformResumed(){
        return mIsResumed;
    }

    @Override
    public void finish() {
        super.finish();
        //之前在onDestroy方法remove会有延时，快速关闭activity时，导致栈的size不准
        mEnv.removeActivity(this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mEnv.removeActivity(this); //做二次remove防止finish没被调用
        getSupportFragmentManager().removeOnBackStackChangedListener(this);
    }

    @Override
    protected void onNewIntent(final Intent intent) {
        super.onNewIntent(intent);

        if(mHandler == null){
            mHandler = new Handler(Looper.getMainLooper());
        }

        mHandler.post(new Runnable() {
            @Override
            public void run() {
                handleIntent(intent);
            }
        });
    }

    public void handleIntent(Intent intent){
        String[] fragmentNames = null;
        if (intent == null) {
            return;
        }else {
            try {
                fragmentNames = intent.getStringArrayExtra(INTENT_EXTRA_FRAGMENT_TAG);
            }catch (Exception exception) {
                return;
            }
        }

        setIntent(intent);

        int[] modes = intent.getIntArrayExtra(INTENT_EXTRA_LAUNCHER_MODE);

        if(fragmentNames != null) {
            for (int i=0; i<fragmentNames.length; ++i) {
                if (!TextUtils.isEmpty(fragmentNames[i])) {
                    BaseFragment fragment = FragmentCenter.popCacheFragment(fragmentNames[i]);
                    if (fragment == null) {
                        return;
                    }
                    pushFragment(fragment, modes[i]);
                    if (fragment.getEnvironment() != null) {
                        setEnvironment(fragment.getEnvironment());
                    }
                }
            }
        }
    }

    @Override
    protected void onStart() {
        super.onStart();
        mIsForeground = true;
    }

    @Override
    protected void onStop() {
        super.onStop();
        mIsForeground = false;
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        ArrayList<String> fragmentIDs = getCurrentFragmentIDs(getSupportFragmentManager());

        if (fragmentIDs !=null && fragmentIDs.size() != 0) {
            outState.putStringArrayList(SAVE_INSTANCE_BUNDLE_KEY_FRAGMENTS,fragmentIDs);
        }

        super.onSaveInstanceState(outState);
    }

    private ArrayList<String> getCurrentFragmentIDs(FragmentManager fragmentManager) {
        ArrayList<String> fragmentIDs = null;
        List<Fragment> fragments = fragmentManager.getFragments();
        if (fragments != null && (fragments.size() != 0)) {
            fragmentIDs = new ArrayList<>();
            for (int i = 0;i<fragments.size();i++) {
                Fragment fragment = fragments.get(i);
                if (fragment != null) {
                    fragmentIDs.add(fragment.getClass().getName());
//                    Log.d(TAG,"Fragment is " + fragment.getClass().getName());
                    ArrayList<String> childFragmentIDs = getCurrentFragmentIDs(fragment.getChildFragmentManager());

                    if (childFragmentIDs != null && (childFragmentIDs.size() != 0)) {
                        fragmentIDs.addAll(childFragmentIDs);
                    }
                }
            }
        }

        return fragmentIDs;
    }

    private void restoreModulesState(Bundle bundle) {
        ArrayList<String> savedFragmentIDs = bundle.getStringArrayList(SAVE_INSTANCE_BUNDLE_KEY_FRAGMENTS);
        if (savedFragmentIDs != null && savedFragmentIDs.size() != 0) {
            if (FragmentCenter.getInstance() != null) {
                for (int i=0; i <savedFragmentIDs.size(); i++) {
                    FragmentCenter.getInstance().fixFragmentModuleState(savedFragmentIDs.get(i));
                }
            }
        }
    }

    /**
     * 压入fragment
     * @param fragment 需要被压入的fragment
     * @param mode
     */
    protected void pushFragment(BaseFragment fragment, int mode) {
        if(fragment == null) {
            return;
        }

        LOG.d(TAG, String.format("pushFragment name=%s, mode=%d", fragment.getClass().getSimpleName(), mode));

        FragmentManager fragmentManager = getSupportFragmentManager();

        if (mode == BaseFragment.LAUNCHER_MODE_START_WITH_POP) {
            // 弹出顶部的Fragment
            fragmentManager.executePendingTransactions();
            removeTopFragment(fragmentManager);
            fragmentManager.popBackStack();
            fragmentManager.executePendingTransactions();
        }

        // 栈顶的Fragment
        BaseFragment topFragment = FragmentHelper.getBackStackTopFragment(fragmentManager);

        if (topFragment != null) {
            LOG.d(TAG, "pushFragment topFragment = " + topFragment.getClass().getSimpleName());
        }

        String fragmentTag = fragment.getClass().getName();

        if (mode == BaseFragment.LAUNCHER_MODE_SINGLE_TOP && topFragment != null
                && fragmentTag.equals(topFragment.getClass().getName())) {
            // 栈顶的Fragment跟目标fragment为同个Fragment
            topFragment.onNewIntent(fragment.getBundleArguments());

            return;
        } else if (mode == BaseFragment.LAUNCHER_MODE_CLEAR_TOP) {
            BaseFragment fragmentInBackstack = (BaseFragment)fragmentManager.findFragmentByTag(fragmentTag);
            if (fragmentInBackstack != null) {
                try {
                    fragmentManager.popBackStack(fragmentTag, 0);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                fragmentInBackstack.onNewIntent(fragment.getBundleArguments());
                return;
            }
        }

        //添加Fragment路径路由打点。
        HashMap<String,String> data = new HashMap<String, String>(1);
        data.put(StatManager.Constant.KEY_FRAGMENT_STACK_OPERATION_TYPE,StatManager.Constant.FRAGMENT_STACK_OPERATION_PUSH);
        Fragment fragment_second = getCurrentFragment();
        if (fragment_second != null) {
            data.put(StatManager.Constant.KEY_FRAGMENT_STACK_TOP_SECOND_FRAGMENT,fragment_second.getClass().getCanonicalName());
        }

        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();

        if (fragment.isUseAnim() && !fragment.isUserActivityAnim()) {
            fragmentTransaction.setCustomAnimations(fragment.mEnterAnimRes, fragment.mExitAnimRes, fragment.mPopEnterAnimRes, fragment.mPopExitAnimRes);
        }

        if (mode == BaseFragment.LAUNCHER_MODE_START_WITH_HIDE && topFragment != null) {
            // Hide Fragment 的时候， 会出发fragment detachFromWindow，一些图片框架会在这个时候隐藏图片
            fragmentTransaction.hide(topFragment);
        }

//        fragmentTransaction.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN);
        fragmentTransaction.add(android.R.id.content, fragment, fragmentTag);
        fragmentTransaction.addToBackStack(fragmentTag);

        //  FragmentTransaction commit 前判断一下Activity是否已经调用过onDestroy
        if (!isFinishing()) {
            fragmentTransaction.commitAllowingStateLoss();
        }

        data.put(StatManager.Constant.KEY_FRAGMENT_STACK_TOP_FIRST_FRAGMENT,fragment.getClass().getCanonicalName());
        Bundle args = fragment.getBundleArguments();
        data.put(StatManager.Constant.KEY_FRAGMENT_SECTION_NAME,args.getString(StatManager.Constant.KEY_FRAGMENT_SECTION_NAME));
        FrameworkService.getInstance().getService(StatManager.class).addStat(StatManager.Constant.CATEGORY_FRAGMENT_PATH,data);

    }

    private void removeTopFragment(FragmentManager fragmentManager) {
        try {
            BaseFragment topFragment = FragmentHelper.getBackStackTopFragment(fragmentManager);
            if (topFragment != null) {
                fragmentManager.beginTransaction()
                        .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_CLOSE)
                        .remove(topFragment)
                        .commitAllowingStateLoss();
            }
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    public void showDialogFragment(DialogFragment diagFragment) {
        if(diagFragment == null) return;

        //添加Fragment路径路由打点。
        HashMap<String,String> data = new HashMap<String, String>(1);
        data.put(StatManager.Constant.KEY_FRAGMENT_STACK_OPERATION_TYPE,StatManager.Constant.FRAGMENT_STACK_OPERATION_PUSH);
        Fragment fragment_second = getCurrentFragment();
        if (fragment_second != null) {
            data.put(StatManager.Constant.KEY_FRAGMENT_STACK_TOP_SECOND_FRAGMENT,fragment_second.getClass().getCanonicalName());
        }

        FragmentTransaction fragAction = getSupportFragmentManager().beginTransaction();
//        // Create and show the dialog.
        diagFragment.show(fragAction, DIALOG_FRAGMENT_TAG);

        //添加Fragment路径路由打点。
        data.put(StatManager.Constant.KEY_FRAGMENT_STACK_TOP_FIRST_FRAGMENT,diagFragment.getClass().getCanonicalName());
        FrameworkService.getInstance().getService(StatManager.class).addStat(StatManager.Constant.CATEGORY_FRAGMENT_PATH,data);
    }

    @Override
    public void onBackPressed() {
        Fragment baseFragment = getCurrentFragment();
        if(baseFragment instanceof BaseFragment){
            if(((BaseFragment)baseFragment).onBackPressed()){
                return;
            }
        }

        popCurrentFragment();
    }

    public void popCurrentFragment(){
         //解决fragment addToBackStack后，按返回键出现空白的Activity问题
        if (getSupportFragmentManager().getBackStackEntryCount() <= 1) {
            BaseFragment topFragment = FragmentHelper.getTopBaseFragment(getSupportFragmentManager());
            doFinish();
            if (topFragment != null && topFragment.isUseAnim()) {
                overridePendingTransition(0, topFragment.mPopExitAnimRes);
            }
        } else {
            try {
                //添加Fragment路径路由打点。
                HashMap<String,String> data = new HashMap<String, String>(1);
                data.put(StatManager.Constant.KEY_FRAGMENT_STACK_OPERATION_TYPE,StatManager.Constant.FRAGMENT_STACK_OPERATION_POP);
                Fragment fragment_first = getCurrentFragment();
                if (fragment_first != null) {
                    data.put(StatManager.Constant.KEY_FRAGMENT_STACK_TOP_FIRST_FRAGMENT,fragment_first.getClass().getCanonicalName());
                }

                super.onBackPressed();

                Fragment fragment_second = getCurrentFragment();
                if (fragment_second != null) {
                    data.put(StatManager.Constant.KEY_FRAGMENT_STACK_TOP_SECOND_FRAGMENT,fragment_second.getClass().getCanonicalName());
                }
                FrameworkService.getInstance().getService(StatManager.class).addStat(StatManager.Constant.CATEGORY_FRAGMENT_PATH,data);

            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }

    /**
     * 弹出栈顶Fragment，当后退栈为1的时候，弹出Fragment后会出现空白的Activity
     */
    public void popCurrentFragmentOnly() {
        if (getSupportFragmentManager().getBackStackEntryCount() > 0) {
            try {
                super.onBackPressed();
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }

    public void doFinish(){
        this.finish();
    }

    public Fragment getCurrentFragment() {
        return FragmentHelper.getTopOriginFragment(getSupportFragmentManager());
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        if (FrameworkFacade.getInstance().getConfig().isOpenStackDebugTool()) {
            new DebugStackTool().attachToActivity(this);
        }
    }

    @Override
    public void onBackStackChanged() {
        int count = getSupportFragmentManager().getBackStackEntryCount();
        LOG.d("hideFragment", String.format("onBackStackChanged activity=%s，backstack count=%d", getClass().getSimpleName(), count));

        if (count > 0) {
            // 栈顶的Fragment
            Fragment topFirst = FragmentHelper.getBackStactFragmentByIndex(getSupportFragmentManager(), count - 1);
            if (topFirst == null) {
                return;
            }

            if (topFirst instanceof BaseFragment) {
                BaseFragment tmpFragment = (BaseFragment) topFirst;
                if (tmpFragment.isCovered()) {
                    tmpFragment.onUncover();
                    tmpFragment.setCovered(false);
                }
            }

            if (count > 1) {
                // 栈顶往下算，第二个Fragment
                Fragment topSecond = FragmentHelper.getBackStactFragmentByIndex(getSupportFragmentManager(), count - 2);
                if (topSecond instanceof BaseFragment && !((BaseFragment) topSecond).isCovered()) {
                    ((BaseFragment) topSecond).onCover();
                    ((BaseFragment) topSecond).setCovered(true);
                }

            }
        }

        if (FrameworkFacade.getInstance().getConfig().isHideDeepFragment()) {
            changeFragmentVisibleState();
        }
    }

    /**
     * 修改Fragment的 hid
     */
    private void changeFragmentVisibleState(){
        int index = getSupportFragmentManager().getBackStackEntryCount() - 1;
        Fragment topFirst = FragmentHelper.getBackStactFragmentByIndex(getSupportFragmentManager(), index);
        if (topFirst == null) {
            return;
        }

        FragmentTransaction fT = getSupportFragmentManager().beginTransaction();
        Fragment topSecond = null;
        Fragment topThird = null;

        if (topFirst.isHidden()) {
            fT.show(topFirst);
        }

        switch (index){
            case 0:
                break;
            case 1:
                // 3 --> 2
                topSecond = FragmentHelper.getBackStactFragmentByIndex(getSupportFragmentManager(), index - 1);
                if (topSecond != null && topSecond.isHidden()){
                    fT.show(topSecond);
                }
                break;
            default:
                topSecond = FragmentHelper.getBackStactFragmentByIndex(getSupportFragmentManager(),index - 1);
                topThird = FragmentHelper.getBackStactFragmentByIndex(getSupportFragmentManager(),index - 2);
                if (topSecond != null && topSecond.isHidden()) {
                    fT.show(topSecond);
                }
                if (topThird != null && !topThird.isHidden()) {
                    fT.hide(topThird);
                }
                break;
        }

        if (!isFinishing()) {
            fT.commitAllowingStateLoss();
        }
    }
}
